@@ -5,8 +5,7 @@
 /* All rights reserved                                                   */
 /*                                                                       */
 /* Use of the link grammar parsing system is subject to the terms of the */
-/* license set forth in the LICENSE file included with this software,    */
-/* and also available at http://www.link.cs.cmu.edu/link/license.html    */
+/* license set forth in the LICENSE file included with this software.    */
 /* This license allows free redistribution and use in source and binary  */
 /* forms, with or without modification, subject to certain conditions.   */
 /*                                                                       */
@@ -15,8 +14,13 @@
 /* stuff for transforming a dictionary entry into a disjunct list */
 
 #include <math.h>
-#include "api.h"
+#include "api-structures.h"
+#include "build-disjuncts.h"
+#include "dict-api.h"
+#include "dict-common.h"
 #include "disjunct-utils.h"
+#include "externs.h"
+#include "word-utils.h"
 #include "utilities.h" /* For Win32 compatibility features */
 
 /* Temporary connectors used while converting expressions into disjunct lists */
@@ -33,8 +37,8 @@ typedef struct clause_struct Clause;
 struct clause_struct
 {
 	Clause * next;
-	float cost;
-	float maxcost;
+	double cost;
+	double maxcost;
 	Tconnector * c;
 };
 
@@ -161,7 +165,7 @@ static Tconnector * build_terminal(Exp * e)
 }
 
 #ifdef UNUSED_FUNCTION
-static xxxfloat-int maxcost_of_expression(Exp *e)
+static xxxdouble-int maxcost_of_expression(Exp *e)
 {
 	E_list * e_list;
 	int m, m1;
@@ -181,7 +185,7 @@ static xxxfloat-int maxcost_of_expression(Exp *e)
  * This returns the maximum maxcost of any disjunct in the sentence
  * Assumes the sentence expressions have been constructed
  */
-static xxx-float-int maxcost_of_sentence(Sentence sent)
+static xxx-double-int maxcost_of_sentence(Sentence sent)
 {
 	X_node * x;
 	int w, m, m1;
@@ -214,8 +218,8 @@ static Clause * build_clause(Exp *e)
 		c1 = (Clause *) xalloc(sizeof (Clause));
 		c1->c = NULL;
 		c1->next = NULL;
-		c1->cost = 0.0f;
-		c1->maxcost = 0.0f ;
+		c1->cost = 0.0;
+		c1->maxcost = 0.0;
 		for (e_list = e->u.l; e_list != NULL; e_list = e_list->next)
 		{
 			c2 = build_clause(e_list->e);
@@ -257,8 +261,8 @@ static Clause * build_clause(Exp *e)
 	{
 		c = (Clause *) xalloc(sizeof(Clause));
 		c->c = build_terminal(e);
-		c->cost = 0.0f;
-		c->maxcost = 0.0f;
+		c->cost = 0.0;
+		c->maxcost = 0.0;
 		c->next = NULL;
 	}
 	else
@@ -361,7 +365,7 @@ static Connector * extract_connectors(Tconnector *e, int c)
  * string is the print name of word that generated this disjunct.
  */
 static Disjunct *
-build_disjunct(Clause * cl, const char * string, float cost_cutoff)
+build_disjunct(Clause * cl, const char * string, double cost_cutoff)
 {
 	Disjunct *dis, *ndis;
 	dis = NULL;
@@ -381,7 +385,7 @@ build_disjunct(Clause * cl, const char * string, float cost_cutoff)
 	return dis;
 }
 
-Disjunct * build_disjuncts_for_X_node(X_node * x, float cost_cutoff)
+Disjunct * build_disjuncts_for_X_node(X_node * x, double cost_cutoff)
 {
 	Clause *c ;
 	Disjunct * dis;
@@ -485,7 +489,7 @@ static unsigned int count_clause(Exp *e)
  */
 unsigned int count_disjunct_for_dict_node(Dict_node *dn)
 {
-	return count_clause(dn->exp);
+	return (NULL == dn) ? 0 : count_clause(dn->exp);
 }
 #endif /* OBSOLETE_MEMORY_PIGGY */
 
@@ -502,9 +506,8 @@ X_node * build_word_expressions(Dictionary dict, const char * s)
 	X_node * x, * y;
 
 	dn_head = dictionary_lookup_list(dict, s);
-	dn = dn_head;
-
 	x = NULL;
+	dn = dn_head;
 	while (dn != NULL)
 	{
 		add_empty_word(dict, dn);
@@ -516,7 +519,7 @@ X_node * build_word_expressions(Dictionary dict, const char * s)
 		x->string = dn->string;
 		dn = dn->right;
 	}
-	free_lookup_list (dn_head);
+	free_lookup_list (dict, dn_head);
 	return x;
 }
 
@@ -524,7 +527,7 @@ X_node * build_word_expressions(Dictionary dict, const char * s)
  * Turn sentence expressions into disjuncts.
  * Sentence expressions must have been built, before calling this routine.
  */
-void build_sentence_disjuncts(Sentence sent, float cost_cutoff)
+void build_sentence_disjuncts(Sentence sent, double cost_cutoff)
 {
 	Disjunct * d;
 	X_node * x;
