@@ -1,11 +1,11 @@
 /*************************************************************************/
 /* Copyright (c) 2004                                                    */
 /* Daniel Sleator, David Temperley, and John Lafferty                    */
+/* Copyright (c) 2013 Linas Vepstas                                      */
 /* All rights reserved                                                   */
 /*                                                                       */
 /* Use of the link grammar parsing system is subject to the terms of the */
-/* license set forth in the LICENSE file included with this software,    */
-/* and also available at http://www.link.cs.cmu.edu/link/license.html    */
+/* license set forth in the LICENSE file included with this software.    */
 /* This license allows free redistribution and use in source and binary  */
 /* forms, with or without modification, subject to certain conditions.   */
 /*                                                                       */
@@ -877,7 +877,7 @@ void conjunction_prune(Sentence sent, Parse_Options opts)
 void init_count(Sentence sent)
 {
 	if (NULL == sent->count_ctxt)
-		sent->count_ctxt = (count_context_t *) malloc (sizeof(count_context_t));
+		sent->count_ctxt = (count_context_t *) xalloc (sizeof(count_context_t));
 	memset(sent->count_ctxt, 0, sizeof(count_context_t));
 
 	init_table(sent);
@@ -888,6 +888,6 @@ void free_count(Sentence sent)
 	if (NULL == sent->count_ctxt) return;
 
 	free_table(sent->count_ctxt);
-	free(sent->count_ctxt);
+	xfree(sent->count_ctxt, sizeof(count_context_t));
 	sent->count_ctxt = NULL;
 }
