@@ -1,11 +1,11 @@
 /*************************************************************************/
 /* Copyright (c) 2004                                                    */
 /* Daniel Sleator, David Temperley, and John Lafferty                    */
+/* Copyright (c) 2009 Linas Vepstas                                      */
 /* All rights reserved                                                   */
 /*                                                                       */
 /* Use of the link grammar parsing system is subject to the terms of the */
-/* license set forth in the LICENSE file included with this software,    */
-/* and also available at http://www.link.cs.cmu.edu/link/license.html    */
+/* license set forth in the LICENSE file included with this software.    */
 /* This license allows free redistribution and use in source and binary  */
 /* forms, with or without modification, subject to certain conditions.   */
 /*                                                                       */
@@ -74,4 +74,30 @@ void prt_error(const char *fmt, ...)
 	va_end(args);
 }
 
+#define MAX_FUNCTION_NAME_SIZE 128
+/**
+ * Check whether the given feature is enabled. It is considered
+ * enabled if it is found in the comma-separated list of features.
+ * This list, if not empty, has a leading and a trailing comma.
+ */
+Boolean feature_enabled(const char * list, const char * feature)
+{
+	char buff[MAX_FUNCTION_NAME_SIZE] = ",";
+	size_t len;
+
+	strncpy(buff+1, feature, sizeof(buff)-2);
+	len = strlen(feature);
+	if (len < sizeof(buff)-2)
+	{
+		buff[len+1] = ',';
+		buff[len+2] = '\0';
+	}
+	else
+	{
+		buff[sizeof(buff)-1] = '\0';
+	}
+	return NULL != strstr(list, buff);
+}
+
+
 /* ============================================================ */
