@@ -4,8 +4,7 @@
 /* All rights reserved                                                    */
 /*                                                                        */
 /* Use of the link grammar parsing system is subject to the terms of the  */
-/* license set forth in the LICENSE file included with this software,     */
-/* and also available at http://www.link.cs.cmu.edu/link/license.html     */
+/* license set forth in the LICENSE file included with this software.     */
 /* This license allows free redistribution and use in source and binary   */
 /* forms, with or without modification, subject to certain conditions.    */
 /*                                                                        */
@@ -122,7 +121,7 @@ void free_fast_matcher(Sentence sent)
 	free_match_list(ctxt->mn_free_list);
 	ctxt->mn_free_list = NULL;
 
-	free(ctxt);
+	xfree(ctxt, sizeof(match_context_t));
 	sent->match_ctxt = NULL;
 }
 
@@ -218,7 +217,7 @@ void init_fast_matcher(Sentence sent)
 	Disjunct * d;
 	match_context_t *ctxt;
 
-	ctxt = (match_context_t *) malloc(sizeof(match_context_t));
+	ctxt = (match_context_t *) xalloc(sizeof(match_context_t));
 	sent->match_ctxt = ctxt;
 
 	ctxt->match_cost = 0;
