@@ -5,8 +5,7 @@
 /* All rights reserved                                                   */
 /*                                                                       */
 /* Use of the link grammar parsing system is subject to the terms of the */
-/* license set forth in the LICENSE file included with this software,    */
-/* and also available at http://www.link.cs.cmu.edu/link/license.html    */
+/* license set forth in the LICENSE file included with this software.    */
 /* This license allows free redistribution and use in source and binary  */
 /* forms, with or without modification, subject to certain conditions.   */
 /*                                                                       */
@@ -14,11 +13,23 @@
 #ifndef _LINKINCLUDESH_
 #define _LINKINCLUDESH_
 
-#include <stdio.h>  /* Needed for FILE* below */
+#include <stdbool.h> /* Needed for bool typedef */
+#include <stdio.h>   /* Needed for FILE* below */
 #include <link-grammar/link-features.h>
 
 LINK_BEGIN_DECLS
 
+#ifndef __bool_true_false_are_defined
+	#ifdef _Bool
+		#define bool                        _Bool
+	#else
+		#define bool                        int
+	#endif
+	#define true                            1
+	#define false                           0
+	#define __bool_true_false_are_defined   1
+#endif
+
 /**********************************************************************
  *
  * System initialization
@@ -44,10 +55,8 @@ link_public_api(Dictionary)
 link_public_api(Dictionary)
      dictionary_create_default_lang(void);
 
-link_public_api(int)
+link_public_api(void)
      dictionary_delete(Dictionary dict);
-link_public_api(int)
-     dictionary_get_max_cost(Dictionary dict);
 
 link_public_api(void)
      dictionary_set_data_dir(const char * path);
@@ -62,7 +71,7 @@ link_public_api(char *)
 
 typedef enum
 {
-	VDAL=1, /* Sort by Violations, Disjunct cost, And cost, Link cost */
+	VDAL=1, /* Sort by Violations, Disjunct cost, Link cost */
 	CORPUS, /* Sort by Corpus cost */
 } Cost_Model_type;
 
@@ -76,18 +85,22 @@ link_public_api(void)
      parse_options_set_verbosity(Parse_Options opts, int verbosity);
 link_public_api(int)
      parse_options_get_verbosity(Parse_Options opts);
+link_public_api(void)
+     parse_options_set_debug(Parse_Options opts, const char * debug);
+link_public_api(char *)
+     parse_options_get_debug(Parse_Options opts);
+link_public_api(void)
+     parse_options_set_test(Parse_Options opts, const char * test);
+link_public_api(char *)
+     parse_options_get_test(Parse_Options opts);
 link_public_api(void)
      parse_options_set_linkage_limit(Parse_Options opts, int linkage_limit);
 link_public_api(int)
      parse_options_get_linkage_limit(Parse_Options opts);
 link_public_api(void)
-     parse_options_set_disjunct_cost(Parse_Options opts, int disjunct_cost);
-link_public_api(void)
-     parse_options_set_disjunct_costf(Parse_Options opts, float disjunct_cost);
-link_public_api(int)
+     parse_options_set_disjunct_cost(Parse_Options opts, double disjunct_cost);
+link_public_api(double)
      parse_options_get_disjunct_cost(Parse_Options opts);
-link_public_api(float)
-     parse_options_get_disjunct_costf(Parse_Options opts);
 link_public_api(void)
      parse_options_set_min_null_count(Parse_Options opts, int null_count);
 link_public_api(int)
@@ -101,12 +114,12 @@ link_public_api(void)
 link_public_api(int)
      parse_options_get_null_block(Parse_Options opts);
 link_public_api(void)
-     parse_options_set_islands_ok(Parse_Options opts, int islands_ok);
-link_public_api(int)
+     parse_options_set_islands_ok(Parse_Options opts, bool islands_ok);
+link_public_api(bool)
      parse_options_get_islands_ok(Parse_Options opts);
 link_public_api(void)
-     parse_options_set_spell_guess(Parse_Options opts, int spell_guess);
-link_public_api(int)
+     parse_options_set_spell_guess(Parse_Options opts, bool spell_guess);
+link_public_api(bool)
      parse_options_get_spell_guess(Parse_Options opts);
 link_public_api(void)
      parse_options_set_short_length(Parse_Options opts, int short_length);
@@ -129,42 +142,42 @@ link_public_api(void)
 link_public_api(Cost_Model_type)
      parse_options_get_cost_model_type(Parse_Options opts);
 link_public_api(void)
-     parse_options_set_use_sat_parser(Parse_Options opts, int use_sat_solver);
-link_public_api(int)
+     parse_options_set_use_sat_parser(Parse_Options opts, bool use_sat_solver);
+link_public_api(bool)
      parse_options_get_use_sat_parser(Parse_Options opts);
 link_public_api(void)
-     parse_options_set_use_viterbi(Parse_Options opts, int use_viterbi);
-link_public_api(int)
+     parse_options_set_use_viterbi(Parse_Options opts, bool use_viterbi);
+link_public_api(bool)
      parse_options_get_use_viterbi(Parse_Options opts);
-link_public_api(int)
+link_public_api(bool)
      parse_options_timer_expired(Parse_Options opts);
-link_public_api(int)
+link_public_api(bool)
      parse_options_memory_exhausted(Parse_Options opts);
-link_public_api(int)
+link_public_api(bool)
      parse_options_resources_exhausted(Parse_Options opts);
 link_public_api(void)
      parse_options_set_screen_width(Parse_Options opts, int val);
 link_public_api(int)
      parse_options_get_screen_width(Parse_Options opts);
 link_public_api(void)
-     parse_options_set_allow_null(Parse_Options opts, int val);
-link_public_api(int)
+     parse_options_set_allow_null(Parse_Options opts, bool val);
+link_public_api(bool)
      parse_options_get_allow_null(Parse_Options opts);
 link_public_api(void)
-     parse_options_set_use_cluster_disjuncts(Parse_Options opts, int val);
-link_public_api(int)
+     parse_options_set_use_cluster_disjuncts(Parse_Options opts, bool val);
+link_public_api(bool)
      parse_options_get_use_cluster_disjuncts(Parse_Options opts);
 link_public_api(void)
-     parse_options_set_display_walls(Parse_Options opts, int val);
-link_public_api(int)
+     parse_options_set_display_walls(Parse_Options opts, bool val);
+link_public_api(bool)
      parse_options_get_display_walls(Parse_Options opts);
 link_public_api(void)
-     parse_options_set_all_short_connectors(Parse_Options opts, int val);
-link_public_api(int)
+     parse_options_set_all_short_connectors(Parse_Options opts, bool val);
+link_public_api(bool)
      parse_options_get_all_short_connectors(Parse_Options opts);
 link_public_api(void)
-     parse_options_set_repeatable_rand(Parse_Options opts, int val);
-link_public_api(int)
+     parse_options_set_repeatable_rand(Parse_Options opts, bool val);
+link_public_api(bool)
      parse_options_get_repeatable_rand(Parse_Options opts);
 link_public_api(void)
      parse_options_reset_resources(Parse_Options opts);
@@ -222,14 +235,14 @@ link_public_api(void)
      parse_options_set_display_links(Parse_Options opts, int val);
 link_public_api(int)
      parse_options_get_display_links(Parse_Options opts);
+link_public_api(void)
+     parse_options_set_display_morphology(Parse_Options opts, int val);
+link_public_api(int)
+     parse_options_get_display_morphology(Parse_Options opts);
 link_public_api(void)
      parse_options_set_display_senses(Parse_Options opts, int val);
 link_public_api(int)
      parse_options_get_display_senses(Parse_Options opts);
-link_public_api(void)
-     parse_options_set_display_suffixes(Parse_Options opts, int val);
-link_public_api(int)
-     parse_options_get_display_suffixes(Parse_Options opts);
 link_public_api(void)
      parse_options_set_echo_on(Parse_Options opts, int val);
 link_public_api(int)
@@ -263,7 +276,7 @@ link_public_api(int)
      sentence_num_linkages_post_processed(Sentence sent);
 link_public_api(int)
      sentence_num_violations(Sentence sent, int i);
-link_public_api(int)
+link_public_api(double)
      sentence_disjunct_cost(Sentence sent, int i);
 link_public_api(int)
      sentence_link_cost(Sentence sent, int i);
@@ -316,6 +329,14 @@ link_public_api(double)
      linkage_get_disjunct_corpus_score(const Linkage linkage, int w);
 link_public_api(const char *)
      linkage_get_word(const Linkage linkage, int w);
+link_public_api(char *)
+     linkage_print_constituent_tree(Linkage linkage, ConstituentDisplayStyle mode);
+link_public_api(void)
+     linkage_free_constituent_tree_str(char *str);
+link_public_api(char *)
+     linkage_print_diagram(const Linkage linkage);
+link_public_api(void)
+     linkage_free_diagram(char * str);
 link_public_api(char *)
      linkage_print_disjuncts(const Linkage linkage);
 link_public_api(void)
@@ -324,25 +345,21 @@ link_public_api(char *)
      linkage_print_links_and_domains(const Linkage linkage);
 link_public_api(void)
      linkage_free_links_and_domains(char *str);
-link_public_api(char *)
-     linkage_print_senses(Linkage linkage);
-link_public_api(void)
-     linkage_free_senses(char *str);
-link_public_api(char *)
-     linkage_print_constituent_tree(Linkage linkage, ConstituentDisplayStyle mode);
-link_public_api(void)
-     linkage_free_constituent_tree_str(char *str);
 link_public_api(char *)
      linkage_print_postscript(Linkage linkage, int mode);
 link_public_api(void)
      linkage_free_postscript(char * str);
 link_public_api(char *)
-     linkage_print_diagram(const Linkage linkage);
+     linkage_print_pp_msgs(Linkage linkage);
 link_public_api(void)
-     linkage_free_diagram(char * str);
+     linkage_free_pp_msgs(char * str);
+link_public_api(char *)
+     linkage_print_senses(Linkage linkage);
+link_public_api(void)
+     linkage_free_senses(char *str);
 link_public_api(int)
      linkage_unused_word_cost(const Linkage linkage);
-link_public_api(int)
+link_public_api(double)
      linkage_disjunct_cost(const Linkage linkage);
 link_public_api(int)
      linkage_link_cost(const Linkage linkage);
@@ -367,29 +384,6 @@ link_public_api(void)
 link_public_api(void)
      linkage_post_process(Linkage, PostProcessor *);
 
-/**********************************************************************
- *
- * Constituent node
- *
- ***********************************************************************/
-
-typedef struct CNode_s CNode;
-
-link_public_api(CNode *)
-     linkage_constituent_tree(Linkage linkage);
-link_public_api(void)
-     linkage_free_constituent_tree(CNode * n);
-link_public_api(const char *)
-     linkage_constituent_node_get_label(const CNode *n);
-link_public_api(CNode *)
-     linkage_constituent_node_get_child(const CNode *n);
-link_public_api(CNode *)
-     linkage_constituent_node_get_next(const CNode *n);
-link_public_api(int)
-     linkage_constituent_node_get_start(const CNode *n);
-link_public_api(int)
-     linkage_constituent_node_get_end(const CNode *n);
-
 /**********************************************************************
  *
  * Internal functions -- do not use these in new code!
@@ -399,9 +393,9 @@ link_public_api(int)
  ***********************************************************************/
 
 link_public_api(void)
-     dict_display_word_expr(Dictionary dict, const char *);
+     dict_display_word_expr(Dictionary dict, const char *, Parse_Options opts);
 link_public_api(void)
-     dict_display_word_info(Dictionary dict, const char *);
+     dict_display_word_info(Dictionary dict, const char *, Parse_Options opts);
 link_public_api(void)
      left_print_string(FILE* fp, const char *, const char *);
 link_public_api(int)
@@ -433,7 +427,8 @@ link_public_api(void)
 /*******************************************************
  *
  * Obsolete functions -- do not use these in new code!
- * XXX TBD: These will all go away in Version 5.0.
+ * XXX TODO: These will all be removed, once the Russian dicts support
+ * conjunctions; Russian is the last remaining user of these functions.
  *
  ********************************************************/
 
@@ -511,22 +506,28 @@ MS_DEPRECATED link_public_api(void)
 MS_DEPRECATED link_public_api(int)
      parse_options_get_display_union(Parse_Options opts) GNUC_DEPRECATED;
 
-/* For languages with regular prefix/suffix structure, this
- * function returns misleading information. Do not use it
- * in new code! use linkage_get_word() instead! */
-MS_DEPRECATED link_public_api(const char *)
-     sentence_get_word(Sentence sent, int wordnum) GNUC_DEPRECATED;
+/**********************************************************************
+ *
+ * Constituent node
+ *
+ ***********************************************************************/
 
-/* Identical to sentence_get_word()
- * XXX TBD: make this go away in Version 5.0. */
-MS_DEPRECATED link_public_api(const char *)
-     sentence_get_nth_word(Sentence sent, int i) GNUC_DEPRECATED;
+typedef struct CNode_s CNode;
 
-/* Who uses this function, anyway? How did this get exported?
- * XXX TBD: make this go away in Version 5.0. */
+MS_DEPRECATED link_public_api(CNode *)
+     linkage_constituent_tree(Linkage linkage) GNUC_DEPRECATED;
+MS_DEPRECATED link_public_api(void)
+     linkage_free_constituent_tree(CNode * n) GNUC_DEPRECATED;
+MS_DEPRECATED link_public_api(const char *)
+     linkage_constituent_node_get_label(const CNode *n) GNUC_DEPRECATED;
+MS_DEPRECATED link_public_api(CNode *)
+     linkage_constituent_node_get_child(const CNode *n) GNUC_DEPRECATED;
+MS_DEPRECATED link_public_api(CNode *)
+     linkage_constituent_node_get_next(const CNode *n) GNUC_DEPRECATED;
 MS_DEPRECATED link_public_api(int)
-     sentence_nth_word_has_disjunction(Sentence sent, int i) GNUC_DEPRECATED;
-
+     linkage_constituent_node_get_start(const CNode *n) GNUC_DEPRECATED;
+MS_DEPRECATED link_public_api(int)
+     linkage_constituent_node_get_end(const CNode *n) GNUC_DEPRECATED;
 
 LINK_END_DECLS
 
