@@ -4,8 +4,7 @@
 /* All rights reserved                                                   */
 /*                                                                       */
 /* Use of the link grammar parsing system is subject to the terms of the */
-/* license set forth in the LICENSE file included with this software,    */
-/* and also available at http://www.link.cs.cmu.edu/link/license.html    */
+/* license set forth in the LICENSE file included with this software.    */
 /* This license allows free redistribution and use in source and binary  */
 /* forms, with or without modification, subject to certain conditions.   */
 /*                                                                       */
@@ -13,8 +12,10 @@
 
 #ifdef USE_FAT_LINKAGES
 
-#include "api.h"
 #include "disjunct-utils.h"
+#include "massage.h"
+#include "structures.h"
+#include "word-utils.h"
 
 /* This file contains the functions for massaging disjuncts of the
    sentence in special ways having to do with conjunctions.
